"""
Chess figures.

Import all figure types into top module.
"""

from bishop import Bishop
from king import King
from knight import Knight
from pawn import Pawn
from queen import Queen
from rook import Rook
